@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA $CurrentPackage @IncludeLibs $ScanFileRE );
 
-$VERSION   = '1.26_001';
+$VERSION   = '1.31_001';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
 @EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
@@ -12,20 +12,25 @@ use Config;
 require Exporter;
 our @ISA = qw(Exporter);
 use constant is_insensitive_fs => (
-    -s $0 
+    -s $0
         and (-s lc($0) || -1) == (-s uc($0) || -1)
         and (-s lc($0) || -1) == -s $0
 );
 
 use version;
-use Cwd ();
 use File::Path ();
 use File::Temp ();
-use File::Spec ();
-use File::Basename ();
 use FileHandle;
 use Module::Metadata;
 
+# NOTE: Keep the following imports exactly as specified, even if the Module::ScanDeps source
+# doesn't reference some of them. See '"use lib" idioms' for the reason.
+use Cwd (qw(abs_path));
+use File::Spec;
+use File::Spec::Functions;
+use File::Basename;
+
+
 $ScanFileRE = qr/(?:^|\\|\/)(?:[^.]*|.*\.(?i:p[ml]|t|al))$/;
 
 =head1 NAME
@@ -172,7 +177,7 @@ returns a reference to it.
     $perl_name = path_to_inc_name($path, $warn)
 
 Assumes C<$path> refers to a perl file and does it's best to return the
-name as it would appear in %INC. Returns undef if no match was found 
+name as it would appear in %INC. Returns undef if no match was found
 and a prints a warning to STDERR if C<$warn> is true.
 
 E.g. if C<$path> = perl/site/lib/Module/ScanDeps.pm then C<$perl_name>
@@ -187,8 +192,8 @@ which to search modules without modifying C<@INC> itself.
 
 =head2 B<$Module::ScanDeps::ScanFileRE>
 
-You can set this global variable to specify a regular expression to 
-identify what files to scan. By default it includes all files of 
+You can set this global variable to specify a regular expression to
+identify what files to scan. By default it includes all files of
 the following types: .pm, .pl, .t and .al. Additionally, all files
 without a suffix are considered.
 
@@ -224,7 +229,7 @@ my %Preload = (
     'AnyDBM_File.pm'                    => [qw( SDBM_File.pm )],
     'AnyEvent.pm'                       => 'sub',
     'Authen/SASL.pm'                    => 'sub',
-    'B/Hooks/EndOfScope.pm'             => 
+    'B/Hooks/EndOfScope.pm'             =>
         [qw( B/Hooks/EndOfScope/PP.pm B/Hooks/EndOfScope/XS.pm )],
     'Bio/AlignIO.pm'                    => 'sub',
     'Bio/Assembly/IO.pm'                => 'sub',
@@ -255,7 +260,7 @@ my %Preload = (
                 _glob_in_inc('Catalyst/DispatchType', 1));
     },
     'Catalyst/Engine.pm'                => 'sub',
-    'CGI/Application/Plugin/Authentication.pm' => 
+    'CGI/Application/Plugin/Authentication.pm' =>
         [qw( CGI/Application/Plugin/Authentication/Store/Cookie.pm )],
     'CGI/Application/Plugin/AutoRunmode.pm' => [qw( Attribute/Handlers.pm )],
     'charnames.pm'                      => \&_unicore,
@@ -269,7 +274,7 @@ my %Preload = (
     'Crypt/Random/Generator.pm'         => sub {
         _glob_in_inc('Crypt/Random/Provider', 1);
     },
-    'Date/Manip.pm'                     => 
+    'Date/Manip.pm'                     =>
         [qw( Date/Manip/DM5.pm Date/Manip/DM6.pm )],
     'Date/Manip/Base.pm'                => sub {
         _glob_in_inc('Date/Manip/Lang', 1);
@@ -290,7 +295,7 @@ my %Preload = (
     'DBIx/Perlish.pm'                   => [qw( attributes.pm )],
     'DBIx/ReportBuilder.pm'             => 'sub',
     'Device/ParallelPort.pm'            => 'sub',
-    'Device/SerialPort.pm'              => 
+    'Device/SerialPort.pm'              =>
         [qw( termios.ph asm/termios.ph sys/termiox.ph sys/termios.ph sys/ttycom.ph )],
     'diagnostics.pm'                    => sub {
         # shamelessly taken and adapted from diagnostics.pm
@@ -312,7 +317,7 @@ my %Preload = (
         ) {
             return $_ if _find_in_inc($_);
         }
-        
+
         for (
               "$archlib/pods/perldiag.pod",
               "$privlib/pods/perldiag-$Config{version}.pod",
@@ -358,6 +363,7 @@ my %Preload = (
         IO/Pipe.pm          IO/Socket.pm        IO/Dir.pm
     )],
     'IO/Socket.pm'                      => [qw( IO/Socket/UNIX.pm )],
+    'IUP.pm'                            => 'sub',
     'JSON.pm'                           => sub {
         # add JSON/PP*.pm, JSON/PP/*.pm
         # and ignore other JSON::* modules (e.g. JSON/Syck.pm, JSON/Any.pm);
@@ -365,7 +371,7 @@ my %Preload = (
         return( grep /^JSON\/(PP|XS)/, _glob_in_inc('JSON', 1) );
     },
     'JSON/MaybeXS.pm'                   => [qw(
-        Cpanel/JSON/XS.pm JSON/XS.pm JSON/PP.pm 
+        Cpanel/JSON/XS.pm JSON/XS.pm JSON/PP.pm
     )],
     'List/MoreUtils.pm'                 => 'sub',
     'List/SomeUtils.pm'                 => 'sub',
@@ -385,7 +391,7 @@ my %Preload = (
     },
     'LWP/Parallel/UserAgent.pm'         => [qw( LWP/Parallel.pm )],
     'LWP/UserAgent.pm'                  => sub {
-        return( 
+        return(
           qw( URI/URL.pm URI/http.pm LWP/Protocol/http.pm ),
           _glob_in_inc("LWP/Authen", 1),
           _glob_in_inc("LWP/Protocol", 1),
@@ -401,6 +407,7 @@ my %Preload = (
     'Module/Pluggable.pm'               => sub {
         _glob_in_inc('$CurrentPackage/Plugin', 1);
     },
+    'Moo.pm'                            => [qw( Class/XSAccessor.pm )],
     'Moose.pm'                          => sub {
         _glob_in_inc('Moose', 1),
         _glob_in_inc('Class/MOP', 1),
@@ -417,6 +424,12 @@ my %Preload = (
     },
     'Module/Implementation.pm'          => \&_warn_of_runtime_loader,
     'Module/Runtime.pm'                 => \&_warn_of_runtime_loader,
+    'Mojo/Util.pm'                      => sub {        # html_entities.txt
+        map { $_->{name} } _glob_in_inc('Mojo/resources', 0)
+    },
+    'Mojo/IOLoop/TLS.pm'                => sub {        # server.{crt,key}
+        map { $_->{name} } _glob_in_inc('Mojo/IOLoop/resources', 0)
+    },
     'Net/DNS/Resolver.pm'               => 'sub',
     'Net/DNS/RR.pm'                     => 'sub',
     'Net/FTP.pm'                        => 'sub',
@@ -484,7 +497,7 @@ my %Preload = (
     },
     'SVN/Core.pm'                       => sub {
         _glob_in_inc('SVN', 1),
-        map $_->{name}, _glob_in_inc('auto/SVN', 0),    # *.so, *.bs files
+        map { $_->{name} } _glob_in_inc('auto/SVN', 0),    # *.so, *.bs files
     },
     'Template.pm'                       => 'sub',
     'Term/ReadLine.pm'                  => 'sub',
@@ -519,18 +532,17 @@ my %Preload = (
         _glob_in_inc('XML/Parser/Encodings', 1),
     },
     'XML/SAX.pm'                        => [qw( XML/SAX/ParserDetails.ini ) ],
+    'XML/Twig.pm'                       => [qw( URI.pm )],      # or URI::File or LWP
+    'XML/Twig/XPath.pm'                 => [qw( XML/XPathEngine.pm XML/XPath.pm )],
     'XMLRPC/Lite.pm'                    => sub {
         _glob_in_inc('XMLRPC/Transport', 1);
     },
     'YAML.pm'                           => [qw( YAML/Loader.pm YAML/Dumper.pm )],
-    'YAML/Any.pm'                       => sub { 
+    'YAML/Any.pm'                       => sub {
         # try to figure out what YAML::Any would have used
         my $impl = eval "use YAML::Any; YAML::Any->implementation;";
-        unless ($@) 
-        { 
-            $impl =~ s!::!/!g; 
-            return "$impl.pm"; 
-        }
+        return _mod2pm($impl) unless $@;
+
         _glob_in_inc('YAML', 1);        # fallback
     },
 );
@@ -588,8 +600,10 @@ sub scan_deps {
     }
     my ($type, $path);
     foreach my $input_file (@{$args{files}}) {
-        if ($input_file !~ $ScanFileRE) {
-            warn "Skipping input file $input_file because it matches \$Module::ScanDeps::ScanFileRE\n" if $args{warn_missing};
+        if ($input_file !~ $ScanFileRE)  {
+            warn "Skipping input file $input_file"
+                 . " because it doesn't match \$Module::ScanDeps::ScanFileRE\n"
+                    if $args{warn_missing};
             next;
         }
 
@@ -616,7 +630,51 @@ sub scan_deps {
         }
     }
 
-    scan_deps_static(\%args);
+    {
+        ## "use lib" idioms
+        #
+        # We want to correctly interprete stuff like
+        #
+        #   use FindBin;
+        #   use lib "$FindBin/../lib";
+        #
+        # Find out what $FindBin::Bin etc would have been if "use FindBin" had been
+        # called in the first file to analyze.
+        #
+        # Notes:
+        # (1) We don't want to reimplement FindBin, hence fake $0 locally (as the path of the
+        #     first file analyzed) and call FindBin::again().
+        # (2) If the caller of scan_deps() itself uses FindBin, we don't want to overwrite
+        #     the value of "their" $FindBin::Bin.
+        #
+        # Other idioms seen sometimes:
+        #
+        # use lib "$ENV{FOO}/path";
+        # use lib File::Spec->catdir($FindBin::Bin, qw[.. qqlib] );
+        # use lib catdir(dirname($0), "perl");
+        # use lib dirname(abs_path($0));
+        #
+        # In order to correctly interprete these, the modules referenced have to be imported.
+
+
+        require FindBin;
+
+        local $FindBin::Bin;
+        local $FindBin::RealBin;
+        local $FindBin::Script;
+        local $FindBin::RealScript;
+
+        my $_0 = $args{files}[0];
+        local *0 = \$_0;
+        FindBin->again();
+
+        our $Bin = $FindBin::Bin;
+        our $RealBin = $FindBin::RealBin;
+        our $Script = $FindBin::Script;
+        our $RealScript = $FindBin::RealScript;
+
+        scan_deps_static(\%args);
+    }
 
     if ($args{execute} or $args{compile}) {
         scan_deps_runtime(
@@ -678,7 +736,7 @@ sub scan_deps_static {
         }else{ # no caching callback given
             @pm = scan_file($file);
         }
-        
+
         foreach my $pm (@pm){
             add_deps(
                      used_by => $key,
@@ -704,7 +762,7 @@ sub scan_deps_static {
 
     # prevent utf8.pm from being scanned
     $_skip->{$rv->{"utf8.pm"}{file}}++ if $rv->{"utf8.pm"};
-   
+
     while ($recurse) {
         my $count = keys %$rv;
         my @files = sort grep { defined $_->{file} && -T $_->{file} } values %$rv;
@@ -756,22 +814,20 @@ sub scan_deps_runtime {
 sub scan_file{
     my $file = shift;
     my %found;
-    my $FH;
-    open $FH, $file or die "Cannot open $file: $!";
+    open my $fh, $file or die "Cannot open $file: $!";
 
     $SeenTk = 0;
     # Line-by-line scanning
   LINE:
-    while (<$FH>) {
-        chomp(my $line = $_);
+    while (my $line = <$fh>) {
+        chomp($line);
         foreach my $pm (scan_line($line)) {
             last LINE if $pm eq '__END__';
 
             if ($pm eq '__POD__') {
-                while (<$FH>) {
-                    last if (/^=cut/);
+                while ($line = <$fh>) {
+                    next LINE if $line =~ /^=cut/;
                 }
-                next LINE;
             }
 
             # Skip Tk hits from Term::ReadLine and Tcl::Tk
@@ -785,7 +841,7 @@ sub scan_file{
             $found{$pm}++;
         }
     }
-    close $FH or die "Cannot close $file: $!";
+    close $fh or die "Cannot close $file: $!";
     return keys %found;
 }
 
@@ -797,11 +853,13 @@ sub scan_line {
     return '__POD__' if $line =~ /^=\w/;
 
     $line =~ s/\s*#.*$//;
-    $line =~ s/[\\\/]+/\//g;
 
   CHUNK:
     foreach (split(/;/, $line)) {
         s/^\s*//;
+        s/^\w+:\s*//;           # remove LABEL:
+        s/^(?:do\s*)?\{\s*//;   # handle single line blocks like 'do { package foo; use xx; }'
+        s/\s*\}$//;
 
         if (/^package\s+(\w+)/) {
             $CurrentPackage = $1;
@@ -812,10 +870,10 @@ sub scan_line {
         if (/^(?:use|require)\s+v?(\d[\d\._]*)/) {
           # include feature.pm if we have 5.9.5 or better
           if (version->new($1) >= version->new("5.9.5")) {
-              # seems to catch 5.9, too (but not 5.9.4)
+            # seems to catch 5.9, too (but not 5.9.4)
             $found{"feature.pm"}++;
-            next CHUNK;
           }
+          next CHUNK;
         }
 
         if (my ($pragma, $args) = /^use \s+ (autouse|if) \s+ (.+)/x)
@@ -824,13 +882,13 @@ sub scan_line {
             # be specified for the "autouse" and "if" pragmas, e.g.
             #   use autouse Module => qw(func1 func2);
             #   use autouse "Module", qw(func1);
-            # To avoid to parse them ourself, we simply try to eval the 
+            # To avoid to parse them ourself, we simply try to eval the
             # string after the pragma (in a list context). The MODULE
             # should be the first ("autouse") or second ("if") element
             # of the list.
             my $module;
-            { 
-                no strict; no warnings; 
+            {
+                no strict; no warnings;
                 if ($pragma eq "autouse") {
                     ($module) = eval $args;
                 }
@@ -838,7 +896,7 @@ sub scan_line {
                     # The syntax of the "if" pragma is
                     #   use if COND, MODULE => ARGUMENTS
                     # The COND may contain undefined functions (i.e. undefined
-                    # in Module::ScanDeps' context) which would throw an 
+                    # in Module::ScanDeps' context) which would throw an
                     # exception. Sneak  "1 || " in front of COND so that
                     # COND will not be evaluated. This will work in most
                     # cases, but there are operators with lower precedence
@@ -848,9 +906,8 @@ sub scan_line {
                 # punt if there was a syntax error
                 return if $@ or !defined $module;
             };
-            $module =~ s{::}{/}g;
-            $found{"$pragma.pm"}++;
-            $found{"$module.pm"}++;
+            $found{_mod2pm($pragma)}++;
+            $found{_mod2pm($module)}++;
             next CHUNK;
         }
 
@@ -861,7 +918,7 @@ sub scan_line {
             foreach my $dir (do { no strict; no warnings; eval $libs }) {
                 next unless defined $dir;
                 my @dirs = $dir;
-                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname" 
+                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname"
                     if $how =~ /lib/;
                 foreach (@dirs) {
                     unshift(@INC, $_) if -d $_;
@@ -876,76 +933,89 @@ sub scan_line {
     return sort keys %found;
 }
 
-# short helper for scan_chunk
+# short helpers for scan_chunk
 my %LoaderRegexp; # cache
-sub _build_loader_regexp {
-    my $loaders = shift;
-    my $prefix = (@_ && $_[0]) ? $_[0].'::' : '';
-   
-    my $loader = join '|', map quotemeta($_), split /\s+/, $loaders;
-    my $regexp = qr/^\s* use \s+ ($loader)(?!\:) \b \s* (.*)/sx;
-    # WARNING: This doesn't take the prefix into account
-    $LoaderRegexp{$loaders} = $regexp;
-    return $regexp
+
+sub _check_loader {
+    my ($chunk, $loaders, $prefix) = @_;
+    $LoaderRegexp{$loaders} ||= _build_loader_regexp($loaders);
+
+    if (my ($loader, $loadee) = $chunk =~ $LoaderRegexp{$loaders}) {
+        return _extract_loader_dependency($loader, $loadee, $prefix);
+    }
+    return;
 }
 
-# short helper for scan_chunk
-sub _extract_loader_dependency {
-    my $loader = shift;
-    my $loadee = shift;
-    my $prefix = (@_ && $_[0]) ? $_[0].'::' : '';
+sub _build_loader_regexp {
+    my ($loaders) = @_;
+    my $alt = join('|', map { quotemeta($_) }
+                            split(/\s+/, $loaders));
+    return qr/^\s* use \s+ ($alt)(?!\:) \b \s* (.*)/sx;
+}
 
-    my $loader_file = $loader;
-    $loader_file =~ s/::/\//;
-    $loader_file .= ".pm";
+sub _extract_loader_dependency {
+    my ($loader, $loadee, $prefix) = @_;
+    $prefix = $prefix ? "${prefix}::" : "";
 
     return [
-        $loader_file,
-        map { my $mod="$prefix$_"; $mod =~ s{::}{/}g; "$mod.pm" }
-        grep { length and !/^q[qw]?$/ and !/-/ }
-        split /[^\w:-]+/, $loadee
+        _mod2pm($loader),
+        map { _mod2pm("$prefix$_") }
+            grep { length and !/^q[qw]?$/ and !/-/ }
+                 split /[^\w:-]+/, $loadee
         #should skip any module name that contains '-', not split it in two
     ];
 }
 
+sub _mod2pm {
+    my $mod = shift;
+    $mod =~ s!::!/!g;
+    return "$mod.pm";
+}
+
 sub scan_chunk {
     my $chunk = shift;
 
     # Module name extraction heuristics {{{
     my $module = eval {
-        $_ = $chunk;
+        local $_ = $chunk;
         s/^\s*//;
 
+        # "if", "while" etc: analyze the expression
+        s/^(?:if|elsif|unless|while|until) \s* \( \s*//x;
+
+        # "eval" with a block: analyze the block
+        s/^eval \s* \{ \s*//x;
+
+        # "eval" with an expression that's a string literal:
+        # analyze the string
+        s/^eval \s+ (?:['"]|qq?\s*\W) \s*//x;
+
         # TODO: There's many more of these "loader" type modules on CPAN!
         # scan for the typical module-loader modules
-        my $loaders = "asa base parent prefork POE encoding maybe only::matching Mojo::Base";
         # grab pre-calculated regexp or re-build it (and cache it)
-        my $loader_regexp = $LoaderRegexp{$loaders} || _build_loader_regexp($loaders);
-        if ($_ =~ $loader_regexp) { # $1 == loader, $2 == loadee
-          my $retval = _extract_loader_dependency($1, $2);
-          return $retval if $retval;
+        if (my $mods = _check_loader(
+            $_, "asa base parent prefork POE encoding maybe only::matching Mojo::Base")) {
+            return $mods;
+        }
+        if (my $mods = _check_loader(
+            $_, "Catalyst", "Catalyst::Plugin")) {
+            return $mods;
         }
 
-        $loader_regexp = $LoaderRegexp{"Catalyst"} || _build_loader_regexp("Catalyst", "Catalyst::Plugin");
-        if ($_ =~ $loader_regexp) { # $1 == loader, $2 == loadee
-          my $retval = _extract_loader_dependency($1, $2, "Catalyst::Plugin");
-          return $retval if $retval;
+        if (/^use \s+ Class::Autouse \b \s* (.*)/sx
+            or /^Class::Autouse \s* -> \s* autouse \s* (.*)/sx) {
+          return [ 'Class/Autouse.pm',
+                   map { _mod2pm($_) }
+                     grep { length and !/^:|^q[qw]?$/ }
+                       split(/[^\w:]+/, $1) ];
         }
 
-        return [ 'Class/Autouse.pm',
-            map { s{::}{/}g; "$_.pm" }
-              grep { length and !/^:|^q[qw]?$/ } split(/[^\w:]+/, $1) ]
-          if /^use \s+ Class::Autouse \b \s* (.*)/sx
-              or /^Class::Autouse \s* -> \s* autouse \s* (.*)/sx;
+        return _mod2pm($1) if /^(?:use|no) \s+ ([\w:]+)/x; # bareword
 
-        return $1 if /^(?:use|no|require) \s+ ([\w:\.\-\\\/\"\']+)/x;
-        return $1
-          if /^(?:use|no|require) \s+ \( \s* ([\w:\.\-\\\/\"\']+) \s* \)/x;
-
-        if (   s/^eval\s+\"([^\"]+)\"/$1/
-            or s/^eval\s*\(\s*\"([^\"]+)\"\s*\)/$1/)
-        {
-            return $1 if /^\s* (?:use|no|require) \s+ ([\w:\.\-\\\/\"\']*)/x;
+        if (s/^(require|do) [\s(]+//x) {
+            return _mod2pm($1)                          # bareword ("require" only)
+                if $1 eq "require" && /^([\w:]+)/;
+            return $_;                                  # maybe string literal?
         }
 
         if (/(<[^>]*[^\$\w>][^>]*>)/) {
@@ -962,16 +1032,17 @@ sub scan_chunk {
             my @mods;
             push @mods, qw( PerlIO.pm PerlIO/encoding.pm Encode.pm ), _find_encoding($1)
                 if $args =~ /:encoding\((.*?)\)/;
-            push @mods, qw( PerlIO.pm PerlIO/via.pm )
-                if $args =~ /:via\(/;
+            while ($args =~ /:(\w+)(?:\((.*?)\))?/g) {
+                push @mods, "PerlIO/$1.pm";
+                push @mods, "Encode.pm", _find_encoding($2) if $1 eq "encoding";
+            }
+            push @mods, "PerlIO.pm" if @mods;
             return \@mods if @mods;
         }
         if (/\b(?:en|de)code\(\s*['"]?([-\w]+)/) {
-            return [qw( Encode.pm ), _find_encoding($1)]; 
+            return [qw( Encode.pm ), _find_encoding($1)];
         }
 
-        return $1 if /\b do \s+ ([\w:\.\-\\\/\"\']*)/x;
-
         if ($SeenTk) {
             my @modules;
             while (/->\s*([A-Z]\w+)/g) {
@@ -993,10 +1064,10 @@ sub scan_chunk {
         }
 
         # Module::Runtime
-        return $1 if /\b(?:require_module|use_module|use_package_optimistically) \s* \( \s* ([\w:"']+)/x;
+        return $_ if s/^(?:require_module|use_module|use_package_optimistically) \s* \( \s*//x;
 
         # Test::More
-        return $1 if /\b(?:require_ok|use_ok) \s* \( \s* ([\w:"']+)/x;
+        return $_ if s/^(?:require_ok|use_ok) \s* \( \s*//x;
 
         return;
     };
@@ -1006,10 +1077,15 @@ sub scan_chunk {
     return unless defined($module);
     return wantarray ? @$module : $module->[0] if ref($module);
 
-    $module =~ s/^['"]//;
-    return unless $module =~ /^\w/;
+    # extract contents from string literals
+    if ($module =~ /^(['"]) (.*?) \1/x) {
+        $module = $2;
+    }
+    elsif ($module =~ s/^qq? \s* (\W)//x) {
+        (my $closing = $1) =~  tr:([{<:)]}>:;
+        $module =~ s/\Q$closing\E.*//;
+    }
 
-    $module =~ s/\W+$//;
     $module =~ s/::/\//g;
     return if $module =~ /^(?:[\d\._]+|'.*[^']|".*[^"])$/;
 
@@ -1018,12 +1094,11 @@ sub scan_chunk {
 }
 
 sub _find_encoding {
+    my ($enc) = @_;
     return unless $] >= 5.008 and eval { require Encode; %Encode::ExtModule };
 
-    my $mod = $Encode::ExtModule{ Encode::find_encoding($_[0])->name }
-      or return;
-    $mod =~ s{::}{/}g;
-    return "$mod.pm";
+    my $mod = eval { $Encode::ExtModule{ Encode::find_encoding($enc)->name } } or return;
+    return _mod2pm($mod);
 }
 
 sub _add_info {
@@ -1036,7 +1111,7 @@ sub _add_info {
     $file = File::Spec->rel2abs($file);
     $file =~ s|\\|\/|go;
 
-    # Avoid duplicates that can arise due to case differences that don't actually 
+    # Avoid duplicates that can arise due to case differences that don't actually
     # matter on a case tolerant system
     if (File::Spec->case_tolerant()) {
         foreach my $key (keys %$rv) {
@@ -1070,7 +1145,7 @@ sub _add_info {
           if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $used_by } @{ $rv->{$module}{used_by} })
              or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($used_by) } @{ $rv->{$module}{used_by} }));
 
-        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}    
+        # We assume here that another _add_info will be called to provide the other parts of $rv->{$used_by}
         push @{ $rv->{$used_by}{uses} }, $module
           if  ( (!File::Spec->case_tolerant() && !grep { $_ eq $module } @{ $rv->{$used_by}{uses} })
              or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($module) } @{ $rv->{$used_by}{uses} }));
@@ -1151,25 +1226,23 @@ sub _find_in_inc {
 }
 
 sub _glob_in_inc {
-    my $subdir  = shift;
-    my $pm_only = shift;
-    my @files;
+    my ($subdir, $pm_only) = @_;
 
     require File::Find;
 
     $subdir =~ s/\$CurrentPackage/$CurrentPackage/;
 
+    my @files;
     foreach my $inc (grep !/\bBSDPAN\b/, @INC, @IncludeLibs) {
         my $dir = "$inc/$subdir";
         next unless -d $dir;
         File::Find::find(
             sub {
-                return unless -f;
+                return unless -f $_;
                 return if $pm_only and !/\.p[mh]$/i;
                 (my $name = $File::Find::name) =~ s!^\Q$inc\E/!!;
-                push @files, $pm_only
-                  ? $name
-                  : { file => $File::Find::name, name => $name };
+                push @files, $pm_only ? $name
+                                      : { file => $File::Find::name, name => $name };
             },
             $dir
         );
@@ -1181,25 +1254,23 @@ sub _glob_in_inc {
 # like _glob_in_inc, but looks only at the first level
 # (i.e. the children of $subdir)
 # NOTE: File::Find has no public notion of the depth of the traversal
-# in its "wanted" callback, so it's not helpful 
+# in its "wanted" callback, so it's not helpful
 sub _glob_in_inc_1 {
-    my $subdir  = shift;
-    my $pm_only = shift;
-    my @files;
+    my ($subdir, $pm_only) = @_;
 
     $subdir =~ s/\$CurrentPackage/$CurrentPackage/;
 
+    my @files;
     foreach my $inc (grep !/\bBSDPAN\b/, @INC, @IncludeLibs) {
         my $dir = "$inc/$subdir";
         next unless -d $dir;
 
-        opendir my $dh, $dir or next; 
+        opendir(my $dh, $dir) or next;
         my @names = map { "$subdir/$_" } grep { -f "$dir/$_" } readdir $dh;
         closedir $dh;
 
-        push @files, $pm_only
-            ? ( grep { /\.p[mh]$/i } @names )
-            : ( map { { file => "$inc/$_", name => $_ } } @names );
+        push @files, $pm_only ? ( grep { /\.p[mh]$/i } @names )
+                              : ( map { { file => "$inc/$_", name => $_ } } @names );
     }
 
     return @files;
@@ -1233,8 +1304,7 @@ sub set_options {
     my $self = shift;
     my %args = @_;
     foreach my $module (@{ $args{add_modules} }) {
-        $module =~ s/::/\//g;
-        $module .= '.pm' unless $module =~ /\.p[mh]$/i;
+        $module = _mod2pm($module) unless $module =~ /\.p[mh]$/i;
         my $file = _find_in_inc($module)
           or _warn_of_missing_module($module, $args{warn_missing}), next;
         $self->{files}{$module} = $file;
@@ -1337,10 +1407,10 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
 ...
 
     # NOTE: When compiling the block will run as the last CHECK block;
-    # when executing the block will run as the first END block and 
+    # when executing the block will run as the first END block and
     # the programs continues.
-    print $ih 
-        $execute ? "END\n" : "CHECK\n", 
+    print $ih
+        $execute ? "END\n" : "CHECK\n",
         <<'...';
 {
     require DynaLoader;
@@ -1356,7 +1426,7 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
     require Config;
     my $dlext = $Config::Config{dlext};
 
-    while (my ($k, $v) = each %_INC)
+    foreach my $k (keys %_INC)
     {
         # NOTES:
         # (1) An unsuccessful "require" may store an undefined value into %INC.
@@ -1367,6 +1437,8 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         #     "Class/MOP/Class/Immutable/Moose/Meta/Class.pm" => "(set by Moose)"
         #     On some architectures (e.g. Windows) Cwd::abs_path() will throw
         #     an exception for such a pathname.
+
+        my $v = $_INC{$k};
         if (defined $v && !ref $v && -e $v)
         {
             $_INC{$k} = Cwd::abs_path($v);
@@ -1381,12 +1453,12 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
     @_INC = grep { !ref $_ } @_INC;
 
     my @dlls = grep { defined $_ && -e $_ } Module::ScanDeps::DataFeed::_dl_shared_objects();
-    my @shared_objects = @dlls; 
-    push @shared_objects, grep { s/\.\Q$dlext\E$/.bs/ && -e $_ } @dlls;
+    my @shared_objects = @dlls;
+    push @shared_objects, grep { -e $_ } map { (my $bs = $_) =~ s/\.\Q$dlext\E$/.bs/; $bs } @dlls;
 
     # write data file
     my $data_file = $ENV{MSD_DATA_FILE};
-    open my $fh, ">", $data_file 
+    open my $fh, ">", $data_file
         or die "Couldn't open $data_file: $!\n";
     print $fh Data::Dumper::Dumper(
     {
@@ -1444,11 +1516,11 @@ BEGIN { my $_0 = $ENV{MSD_ORIGINAL_FILE}; *0 = \$_0; }
         $execute ? @$execute : ());
 
     die $execute
-        ? "SYSTEM ERROR in executing $file @$execute: $rc" 
-        : "SYSTEM ERROR in compiling $file: $rc" 
+        ? "SYSTEM ERROR in executing $file @$execute: $rc"
+        : "SYSTEM ERROR in compiling $file: $rc"
         unless $rc == 0;
-    
-    my $info = do $data_file 
+
+    my $info = do $data_file
         or die "error extracting info from -c/-x file: ", ($@ || "can't read $data_file: $!");
 
     return $info;
@@ -1460,17 +1532,16 @@ sub _info2rv {
 
     my $rv = {};
 
-    my $incs = join('|', sort { length($b) <=> length($a) } 
-                              map { s:\\:/:g; quotemeta($_) } @{ $info->{'@INC'} });
+    my $incs = join('|', sort { length($b) <=> length($a) }
+                              map { s:\\:/:g; s:^(/.*?)/+$:$1:; quotemeta($_) }
+                                  @{ $info->{'@INC'} });
     my $i = is_insensitive_fs() ? "i" : "";
     my $strip_inc_prefix = qr{^(?$i:$incs)/};
 
     require File::Spec;
 
-    while (my ($key, $path) = each %{ $info->{'%INC'} }) {
-        $path =~ s:\\:/:g;
-        $key =~ s:\\:/:g;
-        $key =~ s/$strip_inc_prefix// if File::Spec->file_name_is_absolute($key);
+    foreach my $key (keys %{ $info->{'%INC'} }) {
+        (my $path = $info->{'%INC'}{$key}) =~ s:\\:/:g;
 
         $rv->{$key} = {
             'used_by' => [],
@@ -1513,8 +1584,8 @@ sub _merge_rv {
         my %mark;
         if ($rv->{$key} and _not_dup($key, $rv, $rv_sub)) {
             warn "Different modules for file '$key' were found.\n"
-                . " -> Using '" . _abs_path($rv_sub->{$key}{file}) . "'.\n"
-                . " -> Ignoring '" . _abs_path($rv->{$key}{file}) . "'.\n";
+                . " -> Using '" . abs_path($rv_sub->{$key}{file}) . "'.\n"
+                . " -> Ignoring '" . abs_path($rv->{$key}{file}) . "'.\n";
             $rv->{$key}{used_by} = [
                 grep (!$mark{$_}++,
                     @{ $rv->{$key}{used_by} },
@@ -1547,22 +1618,13 @@ sub _merge_rv {
 sub _not_dup {
     my ($key, $rv1, $rv2) = @_;
     if (File::Spec->case_tolerant()) {
-        return lc(_abs_path($rv1->{$key}{file})) ne lc(_abs_path($rv2->{$key}{file}));
+        return lc(abs_path($rv1->{$key}{file})) ne lc(abs_path($rv2->{$key}{file}));
     }
     else {
-        return _abs_path($rv1->{$key}{file}) ne _abs_path($rv2->{$key}{file});
+        return abs_path($rv1->{$key}{file}) ne abs_path($rv2->{$key}{file});
     }
 }
 
-sub _abs_path {
-    return join(
-        '/',
-        Cwd::abs_path(File::Basename::dirname($_[0])),
-        File::Basename::basename($_[0]),
-    );
-}
-
-
 sub _warn_of_runtime_loader {
     my $module = shift;
     return if $SeenRuntimeLoader{$module}++;
